@@ -1,5 +1,7 @@
 use skiplist::OrderedSkipList;
-use std::marker::PhantomData;
+#[cfg(not(debug_assertions))]
+use std::hint::unreachable_unchecked;
+use std::{fmt, marker::PhantomData, num::NonZeroU32};
 
 /// A growable container for data.
 ///
@@ -79,9 +81,10 @@ impl<T> PackedData<T> {
             }
             None => {
                 let index = self.data.len();
-                self.data.push(Slot::used(0, item));
+                let generation = unsafe { NonZeroU32::new_unchecked(1) };
+                self.data.push(Slot::used(generation, item));
                 Item {
-                    generation: 0,
+                    generation,
                     index,
                     _marker: PhantomData,
                 }
@@ -100,11 +103,16 @@ impl<T> PackedData<T> {
     ///
     /// Panics if such an item is not stored.
     pub fn remove(&mut self, item: Item<T>) -> T {
-        let mut old = Slot::empty(self.data[item.index].generation() + 1);
+        let generation = self.data[item.index]
+            .generation()
+            .get()
+            .checked_add(1)
+            .unwrap_or(1);
+        let mut old = Slot::empty(unsafe { NonZeroU32::new_unchecked(generation) });
         std::mem::swap(&mut old, &mut self.data[item.index]);
         self.holes.insert(item.index);
         match old {
-            Slot::Used((generation, inner_item)) => {
+            Slot::Used(generation, inner_item) => {
                 if generation != item.generation {
                     panic!("The item is not stored!");
                 }
@@ -126,7 +134,54 @@ impl<T> PackedData<T> {
     pub fn get(&self, item: Item<T>) -> &T {
         match self.data.get(item.index) {
             Some(slot) => match slot {
-                Slot::Used((generation, inner_item)) => {
+                Slot::Used(generation, inner_item) => {
+                    if *generation != item.generation {
+                        panic!("The item is not stored!");
+                    }
+                    inner_item
+                }
+                Slot::Empty(_) => panic!("The item is not stored!"),
+            },
+            None => panic!("The item is not stored!"),
+        }
+    }
+
+    /// Returns a reference to an item without any safety checks.
+    ///
+    /// # Arguments
+    ///
+    /// * `item` - ID of the item to be retrieved.
+    ///
+    /// # Safety
+    ///
+    /// `item` has to be stored.
+    #[inline]
+    pub unsafe fn get_unchecked(&self, item: Item<T>) -> &T {
+        #[cfg(debug_assertions)]
+        {
+            self.get(item)
+        }
+
+        #[cfg(not(debug_assertions))]
+        match self.data.get_unchecked(item.index) {
+            Slot::Used(_, inner_item) => inner_item,
+            Slot::Empty(_) => unreachable_unchecked(),
+        }
+    }
+
+    /// Returns a mutable reference to an item.
+    ///
+    /// # Arguments
+    ///
+    /// * `item` - ID of the item to be retrieved.
+    ///
+    /// # Panics
+    ///
+    /// Panics if such an item is not stored.
+    pub fn get_mut(&mut self, item: Item<T>) -> &mut T {
+        match self.data.get_mut(item.index) {
+            Some(slot) => match slot {
+                Slot::Used(generation, inner_item) => {
                     if *generation != item.generation {
                         panic!("The item is not stored!");
                     }
@@ -144,29 +199,28 @@ impl<T> PackedData<T> {
     ///
     /// * `item` - ID of the item to be retrieved.
     ///
-    /// # Panics
+    /// # Safety
     ///
-    /// Panics if such an item is not stored.
-    pub fn get_mut(&mut self, item: Item<T>) -> &mut T {
-        match self.data.get_mut(item.index) {
-            Some(slot) => match slot {
-                Slot::Used((generation, inner_item)) => {
-                    if *generation != item.generation {
-                        panic!("The item is not stored!");
-                    }
-                    inner_item
-                }
-                Slot::Empty(_) => panic!("The item is not stored!"),
-            },
-            None => panic!("The item is not stored!"),
+    /// `item` has to be stored.
+    #[inline]
+    pub unsafe fn get_unchecked_mut(&mut self, item: Item<T>) -> &mut T {
+        #[cfg(debug_assertions)]
+        {
+            self.get_mut(item)
+        }
+
+        #[cfg(not(debug_assertions))]
+        match self.data.get_unchecked_mut(item.index) {
+            Slot::Used(_, inner_item) => inner_item,
+            Slot::Empty(_) => unreachable_unchecked(),
         }
     }
 }
 
-#[derive(PartialEq, Eq, Debug)]
+#[derive(Eq)]
 pub struct Item<T> {
     index: usize,
-    generation: u32,
+    generation: NonZeroU32,
     _marker: PhantomData<T>,
 }
 
@@ -184,24 +238,39 @@ impl<T> Clone for Item<T> {
 
 impl<T> Copy for Item<T> {}
 
+impl<T> PartialEq for Item<T> {
+    fn eq(&self, other: &Self) -> bool {
+        self.index == other.index && self.generation == other.generation
+    }
+}
+
+impl<T> fmt::Debug for Item<T> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Item")
+            .field("index", &self.index)
+            .field("generation", &self.generation)
+            .finish()
+    }
+}
+
 enum Slot<T> {
-    Empty(u32),
-    Used((u32, T)),
+    Empty(NonZeroU32),
+    Used(NonZeroU32, T),
 }
 
 impl<T> Slot<T> {
-    fn used(generation: u32, item: T) -> Self {
-        Self::Used((generation, item))
+    fn used(generation: NonZeroU32, item: T) -> Self {
+        Self::Used(generation, item)
     }
 
-    fn empty(generation: u32) -> Self {
+    fn empty(generation: NonZeroU32) -> Self {
         Self::Empty(generation)
     }
 
-    fn generation(&self) -> u32 {
+    fn generation(&self) -> NonZeroU32 {
         match self {
             Self::Empty(generation) => *generation,
-            Self::Used((generation, _)) => *generation,
+            Self::Used(generation, _) => *generation,
         }
     }
 }
@@ -253,6 +322,37 @@ mod tests {
         }
     }
 
+    #[test]
+    fn test_get_unsafe() {
+        struct Data {
+            value: u64,
+        }
+
+        let mut packed = PackedData::with_max_capacity(100);
+        let a = packed.insert(Data { value: 1 });
+        let b = packed.insert(Data { value: 2 });
+        let c = packed.insert(Data { value: 3 });
+
+        assert_eq!(packed.get(a).value, 1);
+        assert_eq!(packed.get(b).value, 2);
+        assert_eq!(packed.get(c).value, 3);
+
+        packed.get_mut(b).value = 8;
+        assert_eq!(packed.get(a).value, 1);
+        assert_eq!(packed.get(b).value, 8);
+        assert_eq!(packed.get(c).value, 3);
+    }
+
+    #[test]
+    fn test_eq() {
+        struct Something(u32);
+        let mut packed = PackedData::with_max_capacity(2);
+        let item_a = packed.insert(Something(1));
+        let item_b = packed.insert(Something(1));
+        assert_eq!(item_a, item_a);
+        assert_ne!(item_a, item_b);
+    }
+
     #[test]
     #[should_panic]
     fn test_remove_twice_panic() {
@@ -308,4 +408,9 @@ mod tests {
         packed.insert(Something(3));
         packed.get_mut(item);
     }
+
+    #[test]
+    fn test_size() {
+        assert_eq!(std::mem::size_of::<Slot<u64>>(), 16);
+    }
 }
